from pygeai.auth.clients import AuthClient
from pygeai.cli.commands import Command, Option, ArgumentsEnum
from pygeai.cli.commands.builders import build_help_text
from pygeai.cli.texts.help import AUTH_HELP_TEXT
from pygeai.core.common.exceptions import MissingRequirementException, WrongArgumentError
from pygeai.core.utils.console import Console


def show_help():
    """
    Displays help text in stdout
    """
    help_text = build_help_text(auth_commands, AUTH_HELP_TEXT)
    Console.write_stdout(help_text)


def get_oauth2_access_token(option_list: list):
    client_id = None
    username = None
    password = None
    scope = "gam_user_data gam_user_roles"
    for option_flag, option_arg in option_list:
        if option_flag.name == "client_id":
            client_id = option_arg
        if option_flag.name == "username":
            username = option_arg
        if option_flag.name == "password":
            password = option_arg
        if option_flag.name == "scope":
            scope = option_arg

    if not (client_id and username and password):
        raise MissingRequirementException("Cannot obtain Oauth2 access token without client_id, username and password")

    client = AuthClient()
    result = client.get_oauth2_access_token(
        client_id=client_id,
        username=username,
        password=password,
        scope=scope
    )
    Console.write_stdout(f"Authorized projects detail: \n{result}")


get_oauth2_access_token_options = [
    Option(
        "client_id",
        ["--client-id", "--cid"],
        "The client identifier provided by Globant.",
        True
    ),
    Option(
        "username",
        ["--username", "-u"],
        "Username for authentication.",
        True
    ),
    Option(
        "password",
        ["--password", "-p"],
        "Password for authentication.",
        True
    ),
    Option(
        "scope",
        ["--scope", "-s"],
        "Space-separated list of requested scopes. (Optional)",
        True
    ),

]


def get_user_profile_information(option_list: list):
    access_token = None
    for option_flag, option_arg in option_list:
        if option_flag.name == "access_token":
            access_token = option_arg

    client = AuthClient()
    result = client.get_user_profile_information(access_token=access_token)
    Console.write_stdout(f"User profile information: \n{result}")


get_user_profile_information_options = [
    Option(
        "access_token",
        ["--access-token", "--token"],
        "Token obtained with the --get-access-token option",
        True
    ),
]


auth_commands = [
    Command(
        "help",
        ["help", "h"],
        "Display help text",
        show_help,
        ArgumentsEnum.NOT_AVAILABLE,
        [],
        []
    ),
    Command(
        "get_oauth2_access_token",
        ["get-access-token", "gat"],
        "Get Oauth acess token for Globant Enterprise AI instance",
        get_oauth2_access_token,
        ArgumentsEnum.REQUIRED,
        [],
        get_oauth2_access_token_options
    ),
    Command(
        "get_user_profile_information_options",
        ["get-user-information", "get-user-info", "gui"],
        "Retrieve user profile information",
        get_user_profile_information_options,
        ArgumentsEnum.REQUIRED,
        [],
        get_user_profile_information_options
    ),
]



from json import JSONDecodeError

from pygeai import logger
from pygeai.auth.endpoints import GET_USER_PROFILE_INFO, GET_OAUTH2_ACCESS_TOKEN
from pygeai.core.base.clients import BaseClient
from pygeai.core.common.exceptions import InvalidAPIResponseException


class AuthClient(BaseClient):

    def get_oauth2_access_token(
            self,
            client_id: str,
            username: str,
            password: str,
            scope: str = "gam_user_data gam_user_roles"
    ) -> dict:
        """
        Retrieves the list of projects that the user is authorized to access within a specific organization.

        :param client_id: str - The client identifier provided by Globant
        :param username: str - Username for authentication
        :param password: str - Password for authentication
        :param scope: str - Space-separated list of requested scopes
        :return: dict - The API response containing the list of authorized projects in JSON format.
        """
        response = self.api_service.get(
            endpoint=GET_OAUTH2_ACCESS_TOKEN,
            params={
                "client_id": client_id,
                "scope": scope,
                "username": username,
                "password": password
            }
        )
        try:
            return response.json()
        except JSONDecodeError as e:
            logger.error(f"Unable to obtain Oauth2 access token: JSON parsing error (status {response.status_code}): {e}. Response: {response.text}")
            raise InvalidAPIResponseException(f"Unable to  obtain Oauth2 access token: {response.text}")

    def get_user_profile_information(self, access_token: str) -> dict:
        """
        Get user profile information

        :param access_token: str - Token obtained in the POST to /oauth/access_token
        :return: dict - The API response containing the user profile information
        """
        self.api_service.token = access_token
        response = self.api_service.get(endpoint=GET_USER_PROFILE_INFO)
        try:
            return response.json()
        except JSONDecodeError as e:
            logger.error(f"Unable to retrieve user profile information: JSON parsing error (status {response.status_code}): {e}. Response: {response.text}")
            raise InvalidAPIResponseException(f"Unable to retrieve user profile information: {response.text}")



import unittest
from unittest.mock import patch, MagicMock
from json import JSONDecodeError

from pygeai.auth.clients import AuthClient
from pygeai.core.common.exceptions import InvalidAPIResponseException


class TestAuthClient(unittest.TestCase):
    """
    python -m unittest pygeai.tests.auth.test_clients.TestAuthClient
    """

    def setUp(self):
        self.client = AuthClient()
        self.mock_response = MagicMock()

    @patch('pygeai.core.services.rest.ApiService.get')
    def test_get_oauth2_access_token_success(self, mock_get):
        self.mock_response.json.return_value = {"access_token": "token-123", "token_type": "Bearer"}
        mock_get.return_value = self.mock_response
        
        result = self.client.get_oauth2_access_token(
            client_id="client-123",
            username="user@example.com",
            password="password123"
        )
        
        mock_get.assert_called_once()
        call_args = mock_get.call_args
        self.assertEqual(call_args[1]['params']['client_id'], "client-123")
        self.assertEqual(call_args[1]['params']['username'], "user@example.com")
        self.assertEqual(call_args[1]['params']['password'], "password123")
        self.assertEqual(call_args[1]['params']['scope'], "gam_user_data gam_user_roles")
        self.assertEqual(result, {"access_token": "token-123", "token_type": "Bearer"})

    @patch('pygeai.core.services.rest.ApiService.get')
    def test_get_oauth2_access_token_custom_scope(self, mock_get):
        self.mock_response.json.return_value = {"access_token": "token-123"}
        mock_get.return_value = self.mock_response
        
        result = self.client.get_oauth2_access_token(
            client_id="client-123",
            username="user@example.com",
            password="password123",
            scope="custom_scope"
        )
        
        call_args = mock_get.call_args
        self.assertEqual(call_args[1]['params']['scope'], "custom_scope")

    @patch('pygeai.core.services.rest.ApiService.get')
    def test_get_oauth2_access_token_json_decode_error(self, mock_get):
        self.mock_response.json.side_effect = JSONDecodeError("error", "doc", 0)
        self.mock_response.status_code = 401
        self.mock_response.text = "Invalid credentials"
        mock_get.return_value = self.mock_response
        
        with self.assertRaises(InvalidAPIResponseException) as context:
            self.client.get_oauth2_access_token(
                client_id="client-123",
                username="user@example.com",
                password="wrong"
            )
        self.assertIn("Unable to  obtain Oauth2 access token", str(context.exception))

    @patch.object(AuthClient, 'api_service', create=True)
    def test_get_user_profile_information_success(self, mock_api_service):
        self.mock_response.json.return_value = {
            "user_id": "user-123",
            "email": "user@example.com",
            "name": "Test User"
        }
        mock_api_service.get.return_value = self.mock_response
        mock_api_service.token = None
        
        client = AuthClient()
        client.api_service = mock_api_service
        result = client.get_user_profile_information("access-token-123")
        
        mock_api_service.get.assert_called_once()
        self.assertEqual(result, {
            "user_id": "user-123",
            "email": "user@example.com",
            "name": "Test User"
        })

    @patch.object(AuthClient, 'api_service', create=True)
    def test_get_user_profile_information_json_decode_error(self, mock_api_service):
        self.mock_response.json.side_effect = JSONDecodeError("error", "doc", 0)
        self.mock_response.status_code = 401
        self.mock_response.text = "Invalid token"
        mock_api_service.get.return_value = self.mock_response
        mock_api_service.token = None
        
        client = AuthClient()
        client.api_service = mock_api_service
        
        with self.assertRaises(InvalidAPIResponseException) as context:
            client.get_user_profile_information("invalid-token")
        self.assertIn("Unable to retrieve user profile information", str(context.exception))


if __name__ == '__main__':
    unittest.main()
